/**
 * Create an middleware function
 * @function apemanMiddlewareSse
 * @param {string} pattern - URL pattern to apply proxy.
 * @param {function} handler - Handler function for each client
 * @param {Object} [options={}] - Optional settings
 * @returns {function}
 */
'use strict'

const co = require('co')
const debug = require('debug')('apeman:middleware:sse')
const asleep = require('asleep')

const SSEClient = require('sse/lib/sseclient')

/** @lends apemanMiddlewareSse */
function create (pattern, handler, options = {}) {
  let { keep = true } = options
  debug('created')
  let _skip = (url) => pattern && !url.match(pattern)
  debug('created')

  return co.wrap(function * apemanMiddlewareSse (ctx, next) {
    let { query: params = {} } = ctx
    ctx.connected = true
    if (_skip(ctx.url)) {
      yield next()
      return
    }
    let { req, res } = ctx
    let client = new SSEClient(req, res)
    client.params = params
    client.fire = (event, data) => client.send(event, JSON.stringify(data))
    client.initialize()
    try {
      yield Promise.resolve(handler(client, ctx))
    } catch (e) {
      throw e
    }
    while (keep && ctx.connected) {
      yield asleep(0)
    }
  })
}

module.exports = create
