/**
 * @class SSEClientPool
 */
'use strict'

class SSEClientPool {
  constructor () {
    const s = this
    s._clients = []
  }

  /**
   * Invalidate clients
   */
  invalidate () {
    const s = this
    s._clients = s._clients.filter((client) => !client.closed)
  }

  /**
   * Push a client
   * @param client
   */
  push (client) {
    const s = this
    client.on('close', () => {
      client.closed = true
      s.invalidate()
    })
    s._clients.push(client)
    s.invalidate()
  }

  /**
   * Get all clients
   * @returns {*[]}
   */
  all () {
    const s = this
    return [ ...s._clients ]
  }
}

module.exports = SSEClientPool
