/**
 * Test case for create.
 * Runs with mocha.
 */
'use strict'

const create = require('../lib/create.js')
const apemanApi = require('apeman-api')
const apemanApiClient = require('apeman-api/client')
const asleep = require('asleep')
const aport = require('aport')
const assert = require('assert')
const EventSource = require('eventsource')
const co = require('co')

describe('create', function () {
  this.timeout(3000)
  let port, baseUrl
  before(() => co(function * () {
    port = yield aport()
    baseUrl = `http://localhost:${port}`
  }))

  after(() => co(function * () {
  }))

  it('Create', () => co(function * () {
    let server = yield apemanApi({
      port,
      configuration: {
        $api: {
          pingPong: {
            ping () {
              return 'pong'
            }
          },
          $serverMiddlewares: [
            create('/sse/bomb', (client) => co(function * () {
              let count = 10
              while (count >= 0) {
                client.send('tick', JSON.stringify(count))
                yield asleep(100)
                count--
              }
              client.send('boom!')
            }), { keep: false })
          ],
          $pathname: '/api/rpc'
        }
      }

    })
    assert(server)

    let client = apemanApiClient(`${baseUrl}/api/rpc`)
    assert.ok(client)

    {
      let pingPong = yield client.connect('pingPong')
      let pong = yield pingPong.ping()
      assert.equal(pong, 'pong')
    }

    yield asleep(300)

    let eventSource = new EventSource(`${baseUrl}/sse/bomb`)
    eventSource.on('tick', (event) => {
      let { data } = event
      console.log('tick', data)
    })

    yield asleep(500)

    yield server.close()
  }))
})

/* global describe, before, after, it */
